@@ -2,7 +2,7 @@
    +----------------------------------------------------------------------+
    | Zend Engine                                                          |
    +----------------------------------------------------------------------+
-   | Copyright (c) 1998-2018 Zend Technologies Ltd. (http://www.zend.com) |
+   | Copyright (c) Zend Technologies Ltd. (http://www.zend.com)           |
    +----------------------------------------------------------------------+
    | This source file is subject to version 2.00 of the Zend license,     |
    | that is bundled with this package in the file LICENSE, and is        |
@@ -41,22 +41,19 @@ struct _zend_generator_node {
 	uint32_t children;
 	union {
 		HashTable *ht; /* if multiple children */
-		struct { /* if one child */
-			zend_generator *leaf;
-			zend_generator *child;
-		} single;
+		zend_generator *single; /* if one child */
 	} child;
+	/* One generator can cache a direct pointer to the current root.
+	 * The leaf member points back to the generator using the root cache. */
 	union {
-		zend_generator *leaf; /* if > 0 children */
-		zend_generator *root; /* if 0 children */
+		zend_generator *leaf; /* if parent != NULL */
+		zend_generator *root; /* if parent == NULL */
 	} ptr;
 };
 
 struct _zend_generator {
 	zend_object std;
 
-	zend_object_iterator *iterator;
-
 	/* The suspended execution context. */
 	zend_execute_data *execute_data;
 
@@ -89,18 +86,16 @@ struct _zend_generator {
 
 	/* ZEND_GENERATOR_* flags */
 	zend_uchar flags;
-
-	zval *gc_buffer;
-	uint32_t gc_buffer_size;
 };
 
 static const zend_uchar ZEND_GENERATOR_CURRENTLY_RUNNING = 0x1;
 static const zend_uchar ZEND_GENERATOR_FORCED_CLOSE      = 0x2;
 static const zend_uchar ZEND_GENERATOR_AT_FIRST_YIELD    = 0x4;
 static const zend_uchar ZEND_GENERATOR_DO_INIT           = 0x8;
+static const zend_uchar ZEND_GENERATOR_IN_FIBER          = 0x10;
 
 void zend_register_generator_ce(void);
-ZEND_API void zend_generator_close(zend_generator *generator, zend_bool finished_execution);
+ZEND_API void zend_generator_close(zend_generator *generator, bool finished_execution);
 ZEND_API void zend_generator_resume(zend_generator *generator);
 
 ZEND_API void zend_generator_restore_call_stack(zend_generator *generator);
@@ -109,38 +104,28 @@ ZEND_API zend_execute_data* zend_generator_freeze_call_stack(zend_execute_data *
 void zend_generator_yield_from(zend_generator *generator, zend_generator *from);
 ZEND_API zend_execute_data *zend_generator_check_placeholder_frame(zend_execute_data *ptr);
 
-ZEND_API zend_generator *zend_generator_update_current(zend_generator *generator, zend_generator *leaf);
+ZEND_API zend_generator *zend_generator_update_current(zend_generator *generator);
+ZEND_API zend_generator *zend_generator_update_root(zend_generator *generator);
 static zend_always_inline zend_generator *zend_generator_get_current(zend_generator *generator)
 {
-	zend_generator *leaf;
-	zend_generator *root;
-
 	if (EXPECTED(generator->node.parent == NULL)) {
 		/* we're not in yield from mode */
 		return generator;
 	}
 
-	leaf = generator->node.children ? generator->node.ptr.leaf : generator;
-	root = leaf->node.ptr.root;
+	zend_generator *root = generator->node.ptr.root;
+	if (!root) {
+		root = zend_generator_update_root(generator);
+	}
 
-	if (EXPECTED(root->execute_data && root->node.parent == NULL)) {
+	if (EXPECTED(root->execute_data)) {
 		/* generator still running */
 		return root;
 	}
 
-	return zend_generator_update_current(generator, leaf);
+	return zend_generator_update_current(generator);
 }
 
 END_EXTERN_C()
 
 #endif
-
-/*
- * Local variables:
- * tab-width: 4
- * c-basic-offset: 4
- * indent-tabs-mode: t
- * End:
- * vim600: sw=4 ts=4 fdm=marker
- * vim<600: sw=4 ts=4
- */
