@@ -5,7 +5,7 @@
    | This source file is subject to version 3.01 of the PHP license,      |
    | that is bundled with this package in the file LICENSE, and is        |
    | available through the world-wide-web at the following url:           |
-   | http://www.php.net/license/3_01.txt                                  |
+   | https://www.php.net/license/3_01.txt                                 |
    | If you did not receive a copy of the PHP license and are unable to   |
    | obtain it through the world-wide-web, please send a note to          |
    | license@php.net so we can mail you a copy immediately.               |
@@ -31,6 +31,7 @@ PHP_RSHUTDOWN_FUNCTION(date);
 PHP_MINIT_FUNCTION(date);
 PHP_MSHUTDOWN_FUNCTION(date);
 PHP_MINFO_FUNCTION(date);
+ZEND_MODULE_POST_ZEND_DEACTIVATE_D(date);
 
 typedef struct _php_date_obj php_date_obj;
 typedef struct _php_timezone_obj php_timezone_obj;
@@ -65,8 +66,12 @@ static inline php_timezone_obj *php_timezone_obj_from_obj(zend_object *obj) {
 
 #define Z_PHPTIMEZONE_P(zv)  php_timezone_obj_from_obj(Z_OBJ_P((zv)))
 
+#define PHP_DATE_CIVIL   1
+#define PHP_DATE_WALL    2
+
 struct _php_interval_obj {
 	timelib_rel_time *diff;
+	int               civil_or_wall;
 	int               initialized;
 	zend_object       std;
 };
